clc
clear
close all
%% Reading Data form Tables

T1 = readtable('unit1_data.xlsx','PreserveVariableNames',false);  %Reads Unit_1 Data from Excel file
T1A = table2array(T1);
p1i = T1A(find(T1A(:,1)==T1A(1,3)):find(T1A(:,1)==T1A(1,4)),1);
f1i =  T1A(find(T1A(:,1)==T1A(1,3)):find(T1A(:,1)==T1A(1,4)),2);

T2 = readtable('unit2_data.xlsx','PreserveVariableNames',false);     %Reads Unit_2 Data from Excel file
T2A = table2array(T2);
p2i = T2A(find(T2A(:,1)==T2A(1,3)):find(T2A(:,1)==T2A(1,4)) , 1);
f2i =  T2A(find(T2A(:,1)==T2A(1,3)):find(T2A(:,1)==T2A(1,4)),2);

T3 = readtable('unit3_data.xlsx','PreserveVariableNames',false);     %Reads Unit_3 Data from Excel file
T3A = table2array(T3);
p3i = T3A(find(T3A(:,1)==T3A(1,3)):find(T3A(:,1)==T3A(1,4)),1);
f3i =  T3A(find(T3A(:,1)==T3A(1,3)):find(T3A(:,1)==T3A(1,4)),2);

lb = [p1i(1),p2i(1),p3i(1)];  %Units Lower Bounds
ub = [p1i(end),p2i(end),p3i(end)];  %Units Upper Bounds
%%
% lb=[320,300,275];
% ub=[800,1200,1100];
TLD = 800;  %800%2500Total Load Demad in MW
%% Fitting 3rd-degree polynominal to data points
c(1,:)= polyfit(p1i,f1i,3);     %returns the coefficients for a polynomial Cost Function for Unit_1
c(2,:)= polyfit(p2i,f2i,3);     %returns the coefficients for a polynomial Cost Function for Unit_2
c(3,:)= polyfit(p3i,f3i,3);     %returns the coefficients for a polynomial Cost Function for Unit_3

% c=[1.27e-7  ,   9.68e-4     ,   6.95    ,   749.55
%       6.453e-8  ,   7.375e-4    ,   7.051   ,   1285.0
%       9.98e-8    ,  1.04e-3     ,   6.531   ,   1531.0];

dc=c(:,1:end-1);            %returns the coefficients for the incremental Cost Functions
dc(:,1)=dc(:,1).*3;
dc(:,2)=dc(:,2).*2;
%%
dce=dc; % dce is dc matrix with the last column-λ

N=20;   %N = iteration count limit

n=1;        %n= iteration number
m=size(c,1);            % m=Total nmber of thermal units

lambda=4.46;   %lambda = Lagrange multiplier (Lambda)

lambda_a(1)=lambda;
del_lambda=0;   %del_lamda = change in lambda
e=0.01;     %e = iteration tolerance
P=zeros(1,m);
Psum=0;
Pnet=zeros(1,m);

Pmin=lb;    %Pmin & Pmax = minimum and maximum power limits
Pmax=ub;
PD=TLD;     %PD = Power Demand

while(1)
    dce=dc;
    dce(:,end)=dce(:,end)-lambda.*ones(size(dce(:,end)));
    Psum(n)=0;
    lambda_a(n)=lambda;
    r=zeros(m,2);
    for k=1:m
        r(k,:)=roots(dce(k,:));
        if isreal(r(k,1))
        else
            r(k,1)=inf;
        end
        if isreal(r(k,2))
        else
            r(k,2)=inf;
        end
       
    end
    r
   indic=[abs(sum(r(:,1))-PD),abs(sum(r(:,2))-PD)];
   indicn=find(indic==min(indic));
   if(max(size(indicn))==2)
       break
   end
   P(n,1:3)=(r(:,indicn))';
   Psum(n)=sum(P(n,:));
%     for k=1:m
%         r=real(roots(dce(k,:)));
%         if r(1)>Pmin(k)
%             P(n,k)=r(1);
%         else
%             P(n,k)=r(2);
%         end
%          Psum(n)= Psum(n)+P(n,k);
%     end
    if n>N
        break
    end
    Pnet(n)=Psum(n)-PD;
    
    if n==1
        if Psum(n)>PD
            lambda=lambda-lambda/1000;
        else
            lambda=lambda+lambda/1000;
        end
    else
        x1=Psum(n-1);
        x2=Psum(n);
        y1=lambda_a(n-1);
        y2=lambda_a(n);
        lambda=(PD-x1)*((y2-y1)/(x2-x1))+y1;
    end
    if abs(Pnet(n))<e
        break
    end
    n=n+1;
end
cnames=["Iteration","λ","Total Generation (MW)","P"];
%string(array2table(P).Properties.VariableNames)
T = table(array2table((1:n)','VariableNames',{'Iteration'}),array2table((lambda_a'),'VariableNames',{'λ'}),array2table((Psum'),'VariableNames',{'Total Generation (MW)'}),array2table(P)).Variables
disp("Ft=F1(P1)+F2(P2)+F3(P3)=")
disp(polyval(c(1,:),P(end,1))+polyval(c(2,:),P(end,2))+polyval(c(3,:),P(end,3)))
disp("Comparison With the Result of Dynamic Programming:")
disp("F1(400)+F2(100)+F3(300)=")
disp(polyval(c(1,:),400)+polyval(c(2,:),100)+polyval(c(3,:),300))
disp("However")
disp("λ_1=dF1(400),  λ_2=dF2(100),  λ_3=dF3(300)")
disp(num2str(polyval(dc(1,:),400))+"   ,     "+num2str(polyval(dc(2,:),100))+"      ,       "+num2str(polyval(dc(3,:),300)))

xt=50:1:800;
figure
plot(xt,polyval(dc(1,:),xt));
ylabel("λ ($/MWh)")
xlabel("Power Generation (MW)")
grid on;
legend("dF_1/dP_1")
line([400  400 ],[4  polyval(dc(1,:),400) ],'Color','red','LineStyle','-','LineWidth',2)
line([0  400 ],[polyval(dc(1,:),400)  polyval(dc(1,:),400) ],'Color','red','LineStyle','-','LineWidth',2)
yticks([4 4.25 4.5881 6 7 7.5])
axis vis3d;


figure
plot(xt,polyval(dc(2,:),xt));
ylabel("λ ($/MWh)")
xlabel("Power Generation (MW)")
grid on
legend("dF_2/dP_2")
line([100  100 ],[0  polyval(dc(2,:),100) ],'Color','red','LineStyle','-','LineWidth',2)
line([0  100 ],[polyval(dc(2,:),100)  polyval(dc(2,:),100) ],'Color','red','LineStyle','-','LineWidth',2)
yticks([0 5 7.8952 10 15 30])
axis vis3d;

figure
plot(xt,polyval(dc(3,:),xt));
ylabel("λ ($/MWh)")
xlabel("Power Generation (MW)")
legend("dF_3/dP_3")
grid on

line([300  300 ],[4.3  polyval(dc(3,:),300) ],'Color','red','LineStyle','-','LineWidth',2)
line([4.3  300 ],[polyval(dc(3,:),300)  polyval(dc(3,:),300) ],'Color','red','LineStyle','-','LineWidth',2)
yticks([4.3  4.4273 4.48 4.49])
axis vis3d;

figure
x=0:2.5:800;
[X,Y]=meshgrid(x,x);
mesh=X+Y;
X(mesh>800)=NaN;
Y(mesh>800)=NaN;
X(X<100)=NaN;
Y(Y<100)=NaN;
X(X>500)=NaN;
Y(Y>500)=NaN;
X(mesh>=600)=NaN;
Y(mesh>=600)=NaN;

w3rd=polyval(c(1,:),X)+polyval(c(2,:),Y)+polyval(c(3,:),(800-(X+Y)));

surfc(X,Y,w3rd)

xlabel('P1 (MW)')
ylabel('P2 (MW)')
zlabel('Total cost($/h)_3rdOrder_Upper&Lower Constraints')
axis vis3d
hold off

disp("Min point of the (surface)=")
disp(min(w3rd,[],'all'))
disp("P1M at the min. point =")
P1M=X(w3rd==min(w3rd,[],'all'));
disp(P1M)
disp("P2M at the min. point =")
P2M=Y(w3rd==min(w3rd,[],'all'));
disp(P2M)
disp("P3M as the result =")
P3M=800-(P1M+P2M);
disp(P3M)
disp("FtM=F1(P1M)+F2(P2M)+F3(P3M)=")
FtM=polyval(c(1,:),P1M)+polyval(c(2,:),P2M)+polyval(c(3,:),P3M);
disp(FtM)
disp("λ_1=dF1(P1M),  λ_2=dF2(P2M),  λ_3=dF3(P3M)")
Lambda_values=[polyval(dc(1,:),P1M),polyval(dc(2,:),P2M),polyval(dc(3,:),P3M)];
disp(Lambda_values);